"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("../utils/fs");
const cli_ux_1 = require("cli-ux");
const generators_1 = require("../generators");
class CreateCmd extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(CreateCmd);
        const generatorClass = generators_1.default(args.type);
        if (!generatorClass)
            this.error("Invalid project type, allowed type are: " + Object.keys(generators_1.default));
        const generator = new generatorClass(this, args.path, flags);
        try {
            await this.makeProjectDir(args.path);
            await generator.generate();
        }
        catch (error) {
            this.error(error, { exit: false });
            this.error("An error occured, cleaning created files...", { exit: false });
            this.resetState(args.path).catch(err => this.error("An error occured while cleanning project !\n" + err));
        }
    }
    async makeProjectDir(path) {
        try {
            await fs_1.default.mkdir(path);
        }
        catch (err) {
            if (err.code !== "EEXIST")
                throw err;
            await this.deleteExistingDirectory(path);
            await this.makeProjectDir(path);
        }
    }
    async resetState(path) {
        fs_1.default.unlinkFolderSync(path);
    }
    async deleteExistingDirectory(path) {
        this.log("This folder already exist, continue project creation will permanently delete the folder.");
        const answer = await cli_ux_1.default.confirm("Continue ?");
        if (!answer) {
            this.log("Project creation cancelled.");
            this.exit(0);
        }
        fs_1.default.unlinkFolderSync(path);
    }
}
exports.default = CreateCmd;
CreateCmd.description = "Create a new project";
CreateCmd.examples = ["$ projecttools create . nodejs"];
CreateCmd.flags = {
    help: command_1.flags.help({ char: "h" })
};
CreateCmd.args = [
    { name: "path", required: true, description: "Path leading to where the new project will be created" },
    { name: "type", required: true, description: "Type of project (NodeJS, Unity, ...)" },
];
