"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const cli_ux_1 = require("cli-ux");
const fs_1 = require("../utils/fs");
const cmd_1 = require("../utils/cmd");
const gitignore_1 = require("../utils/gitignore");
const generator_1 = require("./generator");
const data_1 = require("../data");
class NodeJSGenerator extends generator_1.default {
    constructor() {
        super(...arguments);
        this.name = "NodeJS";
    }
    async generate() {
        const start = Date.now();
        await this.performNpmInit();
        await this.performGitInit();
        await this.performGitIgnore();
        await this.performEslint();
        await this.performAddDeps();
        this.log("Project creation completed in " + (Date.now() - start) / 1000 + "s");
    }
    async performNpmInit() {
        this.log("Creating package.json...");
        const packageName = await cli_ux_1.default.prompt("Package name", { type: "normal", default: "mypackage" });
        const license = await cli_ux_1.default.prompt("License", { type: "normal", default: "ISC" });
        let git = await cli_ux_1.default.prompt("Git repository", { type: "normal", required: false });
        const author = await cli_ux_1.default.prompt("Author", { type: "normal", default: "unknown" });
        if (git)
            git = {
                "type": "git",
                "url": git
            };
        await fs_1.default.writeFile(path_1.join(this.path, "package.json"), JSON.stringify({
            name: packageName,
            version: "0.0.0",
            description: "",
            main: "index.js",
            scripts: {},
            repository: git || {},
            keywords: [],
            author: author,
            license: license
        }, null, 4));
    }
    async performGitInit() {
        this.log("Initializing git...");
        await cmd_1.default(this.path, "git init");
    }
    async performGitIgnore() {
        this.log("Adding gitignore...");
        const gitignore = await gitignore_1.default("node", "visualstudiocode", "windows", "macOS", "linux");
        await fs_1.default.writeFile(path_1.join(this.path, ".gitignore"), gitignore);
    }
    async performEslint() {
        this.log("Initializing Eslint...");
        await fs_1.default.writeFile(path_1.join(this.path, ".eslintrc.json"), JSON.stringify(data_1.default.eslintNodejs, null, 2));
    }
    async performAddDeps() {
        const dependencies = await cli_ux_1.default.prompt("Add depedencies", { type: "normal", required: false });
        try {
            if (dependencies)
                await cmd_1.default(this.path, `npm i ${dependencies}`);
            else
                this.log("Skipped dependecies");
        }
        catch (error) {
            this.error(error, { exit: false });
            this.error("An error occured while adding dependencies", { exit: false });
        }
    }
}
exports.default = NodeJSGenerator;
